@@ -1,6 +1,6 @@
 import logging
 import time
-from datetime import datetime
+from datetime import datetime  # type: ignore
 from logging import Logger
 from uuid import uuid4
 
@@ -25,7 +25,7 @@ class SQLAlchemyOAuthStateStore(OAuthStateStore):
             metadata,
             metadata,
             Column("id", Integer, primary_key=True, autoincrement=True),
-            Column("state", String, nullable=False),
+            Column("state", String(200), nullable=False),
             Column("expire_at", DateTime, nullable=False),
         )
 
@@ -62,9 +62,7 @@ class SQLAlchemyOAuthStateStore(OAuthStateStore):
         try:
             with self.engine.begin() as conn:
                 c = self.oauth_states.c
-                query = self.oauth_states.select().where(
-                    and_(c.state == state, c.expire_at > datetime.utcnow())
-                )
+                query = self.oauth_states.select().where(and_(c.state == state, c.expire_at > datetime.utcnow()))
                 result = conn.execute(query)
                 for row in result:
                     self.logger.debug(f"consume's query result: {row}")
